import asyncio
from js import document
import math



def desenhar_retangulo(x, y, largura, altura, canvas_id, cor_preenchimento):
    canvas = document.getElementById(canvas_id)
    ctx = canvas.getContext('2d')
    ctx.fillStyle = cor_preenchimento
    ctx.fillRect(x, y, largura, altura)

def converter_graus_para_radianos(graus):
    return graus * math.pi / 180

# Função para rotacionar um ponto em torno do baricentro
def rotacionar_ponto(x, y, x_centro, y_centro, angulo):
    radianos = converter_graus_para_radianos(angulo)
    x_rotacionado =   (x - x_centro) * math.cos(radianos) - (y - y_centro) * math.sin(radianos) + x_centro
    y_rotacionado = - (x - x_centro) * math.sin(radianos) - (y - y_centro) * math.cos(radianos) + y_centro
    return x_rotacionado, y_rotacionado

def desenhar_triangulo(x_baricentro, y_baricentro, raio_circunscrito, cor, id_canvas, angulo=0, proporcao_base=1):
    canvas = document.getElementById(id_canvas)
    ctx = canvas.getContext('2d')

    largura_base = raio_circunscrito * proporcao_base
    altura = math.sqrt( raio_circunscrito ** 2 - (largura_base / 2) ** 2)
    
    # Calcula os vértices do triângulo antes da rotação, PADRÃO APONTADO PARA DIREITA
    vertices = [
        (x_baricentro - altura , y_baricentro + (largura_base / 2)),  # Vértice inferior esquerdo
        (x_baricentro + raio_circunscrito, y_baricentro),  # Vértice superior
        (x_baricentro - altura , y_baricentro - (largura_base / 2)) # Vértice inferior direito
    ]
    
    # Rotaciona os vértices
    vertices_rotacionados = [rotacionar_ponto(x, y, x_baricentro, y_baricentro, angulo) for x, y in vertices]
    
    # Desenha o triângulo com os vértices rotacionados
    ctx.beginPath()
    ctx.moveTo(*vertices_rotacionados[0])
    for x, y in vertices_rotacionados[1:]:
        ctx.lineTo(x, y)
    ctx.closePath()
    
    ctx.fillStyle = cor
    ctx.fill()

# New function to draw a circle
def desenhar_arco(
        x_centro, 
        y_centro, 
        raio, 
        angulo_inicio, 
        angulo_fim, 
        id_canvas, 
        cor_preenchimento,
        cor_contorno,
        largura_contorno
    ):
    canvas = document.getElementById(id_canvas)  # Access the canvas DOM element by its ID.
    ctx = canvas.getContext('2d')  # Get the 2D drawing context of the canvas.
    angulo_inicio_rad = converter_graus_para_radianos(angulo_inicio)
    angulo_fim_rad = converter_graus_para_radianos(angulo_fim)

    ctx.beginPath()  # Begin a new path for the circle.
    ctx.arc(x_centro, y_centro, raio, angulo_inicio_rad, angulo_fim_rad)  # Draw the circle path.
    
    if cor_preenchimento:
        ctx.fillStyle = cor_preenchimento
        ctx.fill() 
    if cor_contorno:
        ctx.strokeStyle = cor_contorno
        ctx.stroke()
    if largura_contorno:
        ctx.lineWidth = largura_contorno
        ctx.stroke()


def desenhar_linha(inicio_x, inicio_y, fim_x, fim_y, id_canvas, cor="black", largura=1, padrao="solid"):
    canvas = document.getElementById(id_canvas)  # Acessa o elemento canvas do DOM pelo seu ID.
    ctx = canvas.getContext('2d')  # Obtém o contexto de desenho 2D do canvas.

    ctx.beginPath()  # Inicia um novo caminho para a linha.
    ctx.moveTo(inicio_x, inicio_y)  # Move o ponto de início da linha para as coordenadas especificadas.
    ctx.lineTo(fim_x, fim_y)  # Desenha a linha até as coordenadas especificadas.

    ctx.strokeStyle = cor  # Define a cor da linha.
    ctx.lineWidth = largura  # Define a largura da linha.

    # Define o padrão da linha.
    if padrao == 'dashed':
        ctx.setLineDash([5, 5])  # Define o padrão de linha tracejada.
    elif padrao == 'dotted':
        ctx.setLineDash([1, 5])  # Define o padrão de linha pontilhada.
    else:
        ctx.setLineDash([])  # Linha sólida.

    ctx.stroke()  # Aplica o desenho da linha.


async def animate():
    x = 0
    y = 50
    width = 100
    height = 50
    while x < 300:
        canvas = document.getElementById('myCanvas')
        ctx = canvas.getContext('2d')
        ctx.clearRect(0, 0, canvas.width, canvas.height)

        draw_rect(x, y, width, height)  # Draw the rectangle.
        draw_triangle(x + 50, y + 100, 60, 30)  # Example of drawing a triangle.
        draw_circle(x + 50, y - 30, 25)  # Example of drawing a circle.

        x += 5
        await asyncio.sleep(0.1)


import unicodedata

NOME_PAINEL_FUNDO = "painelFundo"
NOME_PAINEL_FRENTE = "painelFrente"
NOME_PAINEL_AUXILIAR = "painelAuxiliar"

COR_PRIMARIA = "#9D21FC"
COR_SECUNDARIA = "#34F9FF"
COR_ESCURA = "#555555"
COR_CLARA = "#F5F5F5"

TRADUCAO_PADRAO_DE_LINHA = {
    "tracejada":"dashed",
    "pontilhada":"dotted",
    "solida":"solid",
    "sólida":"solid",
}

TRADUCAO_CORES = {
    "preto":"black",
    "prata":"silver",
    "cinza":"gray",
    "branco":"white",
    "marrom":"maroon",
    "vermelho":"red",
    "roxo":"purple",
    "fucsia":"fuchsia",
    "verde":"green",
    "lima":"lime",
    "oliva":"olive",
    "amarelo":"yellow",
    "azul-marinho":"navy",
    "azul":"blue",
    "verde-água":"teal",
}


def normalizar_palavra(palavra):
    palavra_normalizada = unicodedata.normalize('NFD', palavra)
    return ''.join(char for char in palavra_normalizada if char.isascii())

def traduzir(palavra, dicionario):
    palavra = normalizar_palavra(palavra)
    palavra = palavra.lower()
    
    if palavra in dicionario:
        return dicionario[palavra]
    return palavra


from js import document
from .. import constantes

# Function to create a canvas with specific attributes
def criar_camada(id, width, height):
    painel = document.createElement("canvas")
    painel.setAttribute("id", id)
    painel.width = width
    painel.height = height
    painel.style.position = "absolute"
    painel.style.left = 0
    painel.style.top = 0
    return painel

def apagar_painel(id_painel):
    painel = document.getElementById(id_painel)
    contexto = painel.getContext('2d')
    contexto.clearRect(0, 0, painel.width, painel.height)


def criar_painel(
        largura=500, 
        altura=300,
        nome_painel_fundo=constantes.NOME_PAINEL_FUNDO,
        nome_painel_frente=constantes.NOME_PAINEL_FRENTE,
        nome_painel_auxiliar=constantes.NOME_PAINEL_AUXILIAR,
    ):
    
    div = document.createElement("div")
    div.setAttribute("id", "gameCanvas")
    div.style.setProperty("position", "relative")
    div.style.setProperty("width", f"{largura}px")
    div.style.setProperty("height", f"{altura}px")

    for nome_painel in (
        [nome_painel_fundo] + 
        [nome_painel_auxiliar] + 
        [nome_painel_frente]
    ):
        painel = criar_camada(nome_painel, largura, altura)
        div.appendChild(painel)

    return div
    


import random
from abc import ABC, abstractmethod
import asyncio
from .. import constantes
from .. import desenho

class Labirinto:
    DIRECOES = [(0, -1), (0, 1), (-1, 0), (1, 0)]
    PAREDE = '@'
    CELULA = '_'
    NAO_VISITADO = '.'

    def __init__(self, altura, largura):
        self.altura = altura
        self.largura = largura
        self.labirinto = [[Labirinto.NAO_VISITADO for _ in range(largura)] for _ in range(altura)]
        self.paredes = []
        
        self.entrada = None
        self.saida = None

        self._gerar_labirinto()
        self._processamento_pos_geracao()

        self.representacao = None

    def _contar_celulas_adjacentes(self, parede_aleatoria, tipo_posicao):
        celulas_adjacentes = 0
        h, w = parede_aleatoria
        for dH, dW in Labirinto.DIRECOES:
            if self.labirinto[h+dH][w+dW] == tipo_posicao:
                celulas_adjacentes += 1
        return celulas_adjacentes

    def _gerar_posicao_no_meio(self):
        h = int(random.random() * (self.altura - 2)) + 1
        w = int(random.random() * (self.largura - 2)) + 1
        return h, w

    def _gerar_labirinto(self):
        altura_inicial, largura_inicial = self._gerar_posicao_no_meio()
        self.labirinto[altura_inicial][largura_inicial] = Labirinto.CELULA
        for dH, dW in Labirinto.DIRECOES:
            self.paredes.append([altura_inicial + dH, largura_inicial + dW])
            self.labirinto[altura_inicial + dH][largura_inicial + dW] = Labirinto.PAREDE

        while self.paredes:
            parede_aleatoria = self.paredes[int(random.random()*len(self.paredes))-1]

            for h, w in Labirinto.DIRECOES:
                direcao_encontrada = False
                if 0 < parede_aleatoria[0] < self.altura-1 and 0 < parede_aleatoria[1] < self.largura-1:
                    if self.labirinto[parede_aleatoria[0]+h][parede_aleatoria[1]+w] == Labirinto.NAO_VISITADO and self.labirinto[parede_aleatoria[0]-h][parede_aleatoria[1]-w] == Labirinto.CELULA:
                        direcao_encontrada = True
                        celulas_adjacentes = self._contar_celulas_adjacentes(parede_aleatoria, Labirinto.CELULA)
                        if celulas_adjacentes < 2:
                            self.labirinto[parede_aleatoria[0]][parede_aleatoria[1]] = Labirinto.CELULA
                            for h2, w2 in Labirinto.DIRECOES:
                                if (h2, w2) != (-h, -w):
                                    if self.labirinto[parede_aleatoria[0]+h2][parede_aleatoria[1]+w2] != Labirinto.CELULA:
                                        self.labirinto[parede_aleatoria[0]+h2][parede_aleatoria[1]+w2] = Labirinto.PAREDE
                                    if [parede_aleatoria[0]+h2, parede_aleatoria[1]+w2] not in self.paredes:
                                        self.paredes.append([parede_aleatoria[0]+h2, parede_aleatoria[1]+w2])
                        if parede_aleatoria in self.paredes:
                            self.paredes.remove(parede_aleatoria)

                if direcao_encontrada:
                    break

            if parede_aleatoria in self.paredes:
                self.paredes.remove(parede_aleatoria)

    def _processamento_pos_geracao(self):
        # Transformar células não visitadas em paredes
        for i in range(self.altura):
            for j in range(self.largura):
                if self.labirinto[i][j] == Labirinto.NAO_VISITADO:
                    self.labirinto[i][j] = Labirinto.PAREDE

        # Definir entrada e saída
        for i in range(self.largura):
            if self.labirinto[1][i] == Labirinto.CELULA:
                self.labirinto[0][i] = Labirinto.CELULA
                self.entrada = (0, i)
                break

        for i in range(self.largura-1, 0, -1):
            if self.labirinto[self.altura-2][i] == Labirinto.CELULA:
                self.labirinto[self.altura-1][i] = Labirinto.CELULA
                self.saida = (self.altura-1, i)
                break

    def mostrar_labirinto(self):
        for linha in self.labirinto:
            print(' '.join(linha))


    def eh_parede(self, coordenada):
        h, w = coordenada
        return self.labirinto[h][w] == Labirinto.PAREDE
    
    def mostrar(
            self, 
            camada=constantes.NOME_PAINEL_FUNDO,
            largura_do_tile=25,
        ):
        altura_px = self.altura * largura_do_tile
        largura_px = self.largura * largura_do_tile
        painel = desenho.criar_painel(largura_px, altura_px)

        for i, linha in enumerate(self.labirinto):
            for j, celula in enumerate(linha):
                if celula == Labirinto.PAREDE:
                    desenho.desenhar_retangulo(
                        j * largura_do_tile, 
                        i * largura_do_tile, 
                        largura_do_tile, 
                        largura_do_tile, 
                        camada, 
                        cor_preenchimento=constantes.COR_ESCURA
                    )

    def get_labirinto(self):
        return self.labirinto, self.paredes
    

class Jogador(ABC):
    DIRECOES = {
        'cima': (-1, 0),
        'baixo': (1, 0),
        'direita': (0, 1),
        'esquerda': (0, -1)
    }
    DIRECORES_INV = {v:k for k,v in DIRECOES.items()}

    def __init__(self, labirinto):
        self.labirinto = labirinto
        self.posicao = labirinto.entrada
        self.historico = []

    @abstractmethod
    def mover(self):
        pass

    @abstractmethod
    def redondezas_livres(self):
        pass

    def mostrar_jogador_e_labirinto(self):
        for i, linha in enumerate(self.labirinto.labirinto):
            for j, celula in enumerate(linha):
                if (i, j) == self.posicao:
                    print(self.representacao, end=' ')
                else:
                    print(celula, end=' ')
            print()


class JogadorComBussola(Jogador):

    def __init__(self, labirinto):
        super().__init__(labirinto)
        self.representacao = "X"

    def mover(self, direcao):
        if direcao in JogadorComBussola.DIRECOES:
            dH, dW = JogadorComBussola.DIRECOES[direcao]
            nova_posicao = (self.posicao[0] + dH, self.posicao[1] + dW)

            if not self.labirinto.eh_parede(nova_posicao):
                self.posicao = nova_posicao
                self.historico.append(self.posicao)
                return True
        return False
    
    def redondezas_livres(self):
        redondezas = []
        h, w = self.posicao

        for direcao, (dH, dW) in Jogador.DIRECOES:
            if not self.labirinto.eh_parede((h+dH, w+dW)):
                redondezas.append(direcao)
        return redondezas


class JogadorOrientado(Jogador):
    OPCOES_REPRESENTACAO = {
        'cima': '^',
        'direita': '>',
        'baixo': 'v',
        'esquerda': '<'
    }

    ORIENTACAO_PARA_GRAUS = {
        'cima': 90,
        'direita': 0,
        'baixo': 270,
        'esquerda': 180
    }

    def __init__(self, labirinto):
        super().__init__(labirinto)
        self.orientacao = "baixo"
        self.vetor = Jogador.DIRECOES[self.orientacao]
        self.representacao = JogadorOrientado.OPCOES_REPRESENTACAO[self.orientacao]
        self.redondezas_livres()

    async def virar(self, direcao):
        h, w = self.vetor
        if direcao == 'esquerda':
            self.vetor = (-w, h)
            
        elif direcao == 'direita':
            self.vetor = (w, -h)

        self.orientacao = Jogador.DIRECORES_INV[self.vetor]
        self.representacao = JogadorOrientado.OPCOES_REPRESENTACAO[self.orientacao]
        await asyncio.sleep(0.5)
        self.mostrar()
        self.redondezas_livres()
        
    async def mover(self, passos=1): #async 
        dH, dW = self.vetor
        await asyncio.sleep(0.1)
        for p in range(passos):
            nova_posicao = (self.posicao[0] + dH, self.posicao[1] + dW)
            if not self.labirinto.eh_parede(nova_posicao):
                self.posicao = nova_posicao
                self.historico.append(self.posicao)
                await asyncio.sleep(0.5)
                self.mostrar()
                

        self.redondezas_livres()

    
    def redondezas_livres(self):
        self.redondezas = {}
        dH, dW = self.vetor
        h, w = self.posicao
        self.redondezas["frente"] = not self.labirinto.eh_parede((h+dH, w+dW))
        self.redondezas["esquerda"] = not self.labirinto.eh_parede((h-dW, w+dH))
        self.redondezas["direita"] = not self.labirinto.eh_parede((h+dW, w-dH))
        self.redondezas["trás"] = not self.labirinto.eh_parede((h-dH, w-dW))
        
        return self.redondezas
    
    def esta_livre(self, direcao):
        if direcao in self.redondezas.keys():
            return self.redondezas[direcao]

    def mostrar(
            self,
            camada=constantes.NOME_PAINEL_FRENTE,
            largura_do_tile=25,
        ):
        desenho.apagar_painel(camada)
        h, w = self.posicao
        desenho.desenhar_triangulo(
            x_baricentro=w * largura_do_tile + largura_do_tile/2, 
            y_baricentro=h * largura_do_tile + largura_do_tile/2, 
            raio_circunscrito=(largura_do_tile-5) /2, 
            cor=constantes.COR_PRIMARIA, 
            id_canvas=camada, 
            angulo=self.ORIENTACAO_PARA_GRAUS[self.orientacao],
        )

    
def criar_labirinto_e_jogador():
    lab = Labirinto(10,10)
    p = JogadorOrientado(lab)
    return lab, p

if __name__ == "__main__":
    lab = Labirinto(10,10)
    p = JogadorOrientado(lab)


    for i in range(5):
        redond = p.redondezas_livres()
        if "frente" in redond:
            p.mover()
        else:
            p.virar("esquerda")


from setuptools import setup, find_packages

# Read the contents of your requirements.txt file
with open('requirements.txt') as f:
    required_packages = f.read().splitlines()

setup(
    name='edupils',
    version='0.1.18',
    author='Georges Spyrides',
    author_email='georgesmss@gmail.com',
    packages=find_packages(),
    install_requires=required_packages,
    url='https://github.com/georgesms/edupils',
    license='MIT',
    description='A collection of educational tools for Python',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    keywords='python education highschool pyodide',
    # Additional metadata like description, url, etc.
)



class Força:
    def __init__(self, x, y):
        self.x = x
        self.y = y

class Objeto:
    def __init__(self, x, y, theta, vx, vy, massa):
        self.x = x
        self.y = y
        self.theta = theta
        self.vx = vx
        self.vy = vy
        self.massa = massa
        self.forças = []

    def adiciona_força(self, força):
        self.forças.append(força)
    
    def aceleração_resultante(self):
        ax = sum(força.x for força in self.forças) / self.massa
        ay = sum(força.y for força in self.forças) / self.massa
        return ax, ay

    def atualiza_estado(self, dt):
        ax, ay = self.aceleração_resultante()
        self.vx += ax * dt
        self.vy += ay * dt
        self.x += self.vx * dt
        self.y += self.vy * dt
        # Resetar as forças após a atualização
        self.forças = []

class Círculo(Objeto):
    def __init__(self, x, y, theta, vx, vy, massa, raio):
        super().__init__(x, y, theta, vx, vy, massa)
        self.raio = raio

    def detecção_de_colisão(self):
        pass  # Implementar lógica de detecção de colisão

    def baricentro(self):
        return self.x, self.y

class Retângulo(Objeto):
    def __init__(self, x, y, theta, vx, vy, massa, largura, altura):
        super().__init__(x, y, theta, vx, vy, massa)
        self.largura = largura
        self.altura = altura

    def detecção_de_colisão(self):
        pass  # Implementar lógica de detecção de colisão

    def baricentro(self):
        return self.x + self.largura / 2, self.y + self.altura / 2

class Polyline(Objeto):
    def __init__(self, x, y, theta, vx, vy, massa, pontos):
        super().__init__(x, y, theta, vx, vy, massa)
        self.pontos = pontos  # Lista de tuplas (x, y) representando os vértices

    def detecção_de_colisão(self):
        pass  # Implementar lógica de detecção de colisão

    def baricentro(self):
        pass  # Implementar cálculo de baricentro
